import {Toast} from 'antd-mobile';

class AttUtil {

  /**
   * 处理错误
   * @param e 错误对象或者错误消息
   * @param customHandler 自定义处理函数
   */
  static onError(e, customHandler) {
    console.error(e);
    const errorMsg = e.message || e;
    customHandler ? customHandler(errorMsg) : '';
  }

  /**
   * 提醒弹窗延时，单位：秒
   */
  static TOAST_DURATION = 2;

  /**
   * 图片上传默认上限
   */
  static PIC_MAX_NUM = 4;

  /**
   * 图片base64前缀
   */
  static PIC_PREFIX = 'data:image/png;base64,';

  /**
   * 视频base64前缀
   */
  static VIDEO_PREFIX = 'data:video/mp4;base64,';

  /**
   * 音频base64前缀
   */
  static AUDIO_PREFIX = 'data:audio/amr;base64,';

  /**
   * 音频状态码
   */
  static MEDIA_NONE = 0; //音频为空
  static MEDIA_STARTING = 1; //开始
  static MEDIA_RUNNING = 2; //播放中
  static MEDIA_PAUSED = 3; //暂停
  static  MEDIA_STOPPED = 4; //停止

  /**
   * 判断是否为base64编码数据
   */
  static isBase64Data(str) {
    let flag = false;
    if(str.indexOf('data:') !== -1 && str.indexOf('base64') !== -1){
      flag = true;
    }
    return flag
  }

  /**
   * 判断是否为存在元素的数组
   */
  static isFilledArray(obj) {
    return Array.isArray(obj) && obj.length
  }

  /**
   * 获取图片
   * @param sourceType 图片源类型 Camera.PictureSourceType.CAMERA:相机；Camera.PictureSourceType.PHOTOLIBRARY:相册
   * @param onSucc(imageData) 成功回调(图片base64编码数据)
   * @param onFail(errorMsg) 失败回调(错误消息)
   */
  static capturePic(sourceType, onSucc, onFail) {
    try {
      const options = {
        sourceType: sourceType,
        destinationType: Camera.DestinationType.DATA_URL,
        encodingType: Camera.EncodingType.PNG,
        mediaType: Camera.MediaType.PICTURE,
        saveToPhotoAlbum: true,
        quality: 40,
        targetWidth: 1366,
        targetHeight: 768
      };
      const succCb = (data) => {
        onSucc(data);
      };
      const failCb = (msg) => {
        if (msg !== 'No Image Selected') {
          onFail ? onFail(msg) : console.error(msg);
        }
      };
      navigator.camera.getPicture(succCb, failCb, options)
    } catch (e) {
      onFail ? onFail(e) : console.error(e);
    }
  }

  /**
   * 拍照选择
   * @param onSucc(imageData) 成功回调(图片base64编码数据)
   * @param onFail(errorMsg) 失败回调(错误消息)
   */
  static capturePicByCamera(onSucc, onFail) {
    this.capturePic(Camera.PictureSourceType.CAMERA, onSucc, onFail);
  }

  /**
   * 相册选择
   * @param onSucc(base64data) 成功回调(照片base64数据)
   * @param onFail(errorMsg) 失败回调(错误消息)
   */
  static capturePicByAlbum(onSucc, onFail) {
    this.capturePic(Camera.PictureSourceType.PHOTOLIBRARY, onSucc, onFail);
  }

  /**
   * 视频压缩
   * @param onSucc 成功回调
   * @param onFail(errorMsg) 失败回调(错误消息)
   */
  static compressVideo(path, localURL, onSucc, onFail) {
    let index = path.lastIndexOf("\/");
    let fileName = path.substring(index + 1, path.length - 4);
    Toast.loading(`视频处理中`);
    VideoEditor.transcodeVideo(
      (result) => {
        window.resolveLocalFileSystemURL(`file://${result}`, (entry) => {
          entry.file((file) => {
            let reader = new FileReader();
            reader.onload = (evt) => {
              Toast.hide();
              let data = evt.target.result;
              data = data.substring(data.lastIndexOf("base64") + 7, data.length);
              onSucc(data, localURL);
            };
            reader.readAsDataURL(file);
          }, (error) => {
            Toast.hide();
            AttUtil.onError(error, onFail);
          })
        })
      }, (err) => {
        console.log('videoTranscodeError, err: ' + err);
        onFail ? onFail() : '';
      }, {
        fileUri: path,
        outputFileName: fileName,
        outputFileType: VideoEditorOptions.OutputFileType.MPEG4,
        optimizeForNetworkUse: VideoEditorOptions.OptimizeForNetworkUse.YES,
        saveToLibrary: true,
        maintainAspectRatio: true,
        width: 640,
        height: 640,
        videoBitrate: 1000000, // 1 megabit
        audioChannels: 2,
        audioSampleRate: 44100,
        audioBitrate: 128000, // 128 kilobits
        progress: (info) => {
          console.log('transcodeVideo progress callback, info: ' + info);
        }
      }
    );
  }

  /**
   * 录像
   * @param onSucc(base64data) 成功回调(视频base64数据)
   * @param onFail(errorMsg) 失败回调(错误消息)
   */
  static captureVideo(onSucc, onFail) {
    try {
      const options = {
        limit: 1,
        duration: 10,
        quality: 1
      };
      const succCb = (files) => {
        const file = files[0];
        const path = file.fullPath;
        const localURL = file.localURL;
        AttUtil.compressVideo(path, localURL, onSucc, onFail)
      };
      const failCb = (error) => {
        if (error.code === 3) {
          console.info('用户取消拍摄操作');
          return;
        }
        AttUtil.onError(error, onFail);
      };
      navigator.device.capture.captureVideo(succCb, failCb, options);
    } catch (e) {
      AttUtil.onError(e, onFail);
    }
  }

  /**
   * 录音
   */
  static captureAudio(onSucc, onFail) {
    try {
      const options = {
        limit: 1
      };
      const succCb = (files) => {
        const file = files[0];
        const path = file.fullPath;
        const localURL = file.localURL;
        window.resolveLocalFileSystemURL(path, (entry) => {
          entry.file((audio) => {
            const reader = new FileReader();
            reader.onload = (evt) => {
              onSucc(evt.target.result, localURL);
            };
            reader.readAsDataURL(audio);
          }, (error) => {
            AttUtil.onError(error, onFail);
          })
        })
      };
      const failCb = (error) => {
        if (error.code === 3) {
          console.info('用户取消录音操作');
          return;
        }
        AttUtil.onError(error, onFail);
      };
      navigator.device.capture.captureAudio(succCb, failCb, options);
    } catch (e) {
      AttUtil.onError(e, onFail);
    }
  }
}

export default AttUtil;
